# Stuart Macdonald
# sjmac@ku.edu


# INSTRUCTIONS
#
# 1. Go to foot of file and paste in all functions.




# NOTES
#
# 1. This analyzes an experiment to compare sleep phenotypes in inbred DSPR lines and
#    outbred, heterozygous F1 genotypes resulting from DSPR line intercrosses. The
#    principal output is the DSPR version of Supplementary Figure S3.
#
# 2. Input files in the "DAMinitiation", "DAMoutput", and "RILduplication" directories
#    are basically the same as those for the "DSPRF1_Phenotypes_QTL" material. Only here
#    the names of the genotypes specifically define the female and male parents of the
#    genotype using the appropriate DSPR RIL ID. For instance, "22061F.11222M" means the
#    test genotype is a resulting of intercrossing females from 22061 and males from
#    11222. While "11221F.11221M" means the test genotype is 11221.




##########################################################################################
### GLOBAL VARIABLES
##########################################################################################
### To analyze the raw data in the "RILduplication", "DAMinitiation", and "DAMoutput"
### directories, only the "uberDir" object needs to be reset for your system.
##########################################################################################

# DIRECTORIES
# -> Upper-level
# -> RIL duplication files
# -> Monitor initiation files
# -> Monitor output directories
# -> Processed data OUTPUT
# -> Summary data OUTPUT
uberDir <- "/Users/sjm/Desktop/InbredOutbred_DSPR/"
dirRILdup <- paste(uberDir,"RILduplication/",sep="")
dirInit <- paste(uberDir,"DAMinitiation/",sep="")
dirOut <- paste(uberDir,"DAMoutput/",sep="")
dirProcessed <- paste(uberDir,"ProcessedData/",sep="")
dirSummary <- paste(uberDir,"SummaryData/",sep="")

# NAMING SCHEME
# -> Stem for initiation files
# -> Stem for monitor output directories
stemInitFiles <- "daminitiation"
stemDAMdir <- "damoutput"

# LOAD/VALIDATE MONITOR FILES
# -> Set monitor loading format
#    "2halves" = A/B side loading
#    "4quadrants" = 1/2/3/4 quandrant loading
# -> Set # columns, and whether to strip any
# -> Set monitor reading interval
# -> Set error file
# -> Set ad hoc reporting file
monitorFormat <- "4quadrants"
monitorFileNCols <- 42
stripCols <- "no"
readInterval <- 1
readIntUnits <- "mins"
ErrorFile <- paste(uberDir,"MonitorErrors.txt",sep="")
write(paste("Dir","Monitor","LessThanInterval","DoubleInterval",
    "MoreThanDouble",sep="\t"),ErrorFile,append=F)
AdHocReporting <- paste(dirSummary,"AdHocReport.txt",sep="")
write(paste("Directory","Monitor","DateAtStart","TimeAtStart",
	"DateAtEnd","TimeAtEnd","ExptPeriodDays",sep="\t"),
	AdHocReporting,append=F)

# THRESHOLDS
# -> activityThresh - min number activity
#    counts for usable tube
# -> aliveThresh - max number live flies
#    for a genotype at end of run
# -> flyThresh - min number tested flies
#    per genotype
# DETERMINE WHEN FLY IS DEAD
# -> MaxNumSequentialInactiveBins - acceptable
#    number zero activity bins between active
#    bins (720min = 12hrs)
# -> MaxNumActiveBins - number active bins
#    allowed following an inactive period and
#    still consider fly dead at start of that
#    inactive period
# DETERMINE IF FLY ALIVE AT END OF RUN
# -> EndTestWindow - test period at end of run
#    to examine (120min = 2hrs)
# -> EndActivityCount - acceptable number of
#    counts to still consider fly dead
activityThresh <- 100
aliveThresh <- 0
flyThresh <- 7
MaxNumSequentialInactiveBins <- 720
MaxNumActiveBins <- 5
EndTestWindow <- 120
EndActivityCount <- 5



##########################################################################################
### PROCESS DATA
##########################################################################################
### Collects information on monitors initiated and crosses set up. Rips through genotype-
### by-genotype and extracts activity/sleep data on every genotype in a consistent format
### and deposits in the "ProcessedData" directory.
##########################################################################################

# GATHER data on every monitor initiated in experiment
MonitorInfo <- CollateInitiationFiles(dirInit,stemInitFiles,stemDAMdir,monitorFormat)

# GATHER data on all RIL duplications
ril2arb <- ReadInFiles.func(dirRILdup)

# LOOP through all monitors
for(i in 1:nrow(MonitorInfo)) {

    # INPUT activity monitor file
    # VALIDATE the file
    # REMOVE (to NAs) data from tubes that did not get filled
    # REMOVE (to NAs) data from tubes with low activity
    monitorFname <- paste(dirOut,MonitorInfo[i,1],"/",MonitorInfo[i,2],sep="")
    MonDat <- InputMonitorFile(monitorFname,monitorFileNCols,stripCols)
    ValidDat <- CheckDupsIntervalsGaps(MonDat,readInterval,readIntUnits,MonitorInfo[i,],ErrorFile)
    ValidDat <- RemoveBadTubes(ValidDat,MonitorInfo[i,])
    ValidDat <- RemoveLowActivityTubeData(ValidDat,FindLowActivityTubes(ValidDat,activityThresh))

	# TRIM the file to remove non-data collection lines from start and end
	ValidDat <- TrimStartMonitorFile(ValidDat)
	ValidDat <- TrimEndMonitorFile(ValidDat)

    # POPULATE a matrix with details (start time, genotype, experimenter) of
    #   fly in each tube directly from 'MonitorInfo'
    TubeDetailsDat <- PopulateIDMatrix(MonitorInfo[i,],monitorFormat)

    # GET actual RIL barcodes
    RIL <- unlist(lapply(TubeDetailsDat[,"Genotype"],function(X) FindRILBarcode(X,ril2arb)))
    uniqueRIL <- sort(unique(RIL))

    # CALCULATE time of death for each tube in monitor
    DeathDat <- DeathTimes(ValidDat,readInterval,readIntUnits,
        MaxNumSequentialInactiveBins,MaxNumActiveBins)

    # USE 'row of death' column in 'DeathDat' to NA all values in each monitor
    #   column of 'ValidDat' some period of time before a fly dies (and all
    #   timepoints after fly dies)
    PreDeathHrs <- 12
    ResetDat <- ResetAfterDeathActivityCounts(ValidDat,DeathDat,PreDeathHrs)

    # GET Zeitgeber timestamps
    zbTime <- GetZietgeberTime(ResetDat,LightZeitgeber.H,
        LightZeitgeber.M,DarkZeitgeber.H,DarkZeitgeber.M)

	# TRUNCATE dataset to only include whole number of
	#   days (where a full day is a full light + a full
	#   dark period)
	#   STARTS: At first '0' in zbTime
	#           Currently this is 8am. Assuming no monitors
	#             filled before 9am on initiation day, expt.
	#             will start at 8am the day after initiation.
	#   ENDS: A specific number of days after start
	NumExptDays <- 4
	StartRow <- match(0,zbTime[,1])
	EndRow <- StartRow+(1440*NumExptDays)-1
	ResetDat <- ResetDat[StartRow:EndRow,]
	zbTime <- zbTime[StartRow:EndRow,]
	
	# REPORT
	# Directory, Monitor, First/Last data reads in experimental period
	#
	# POINT: To ensure that extracting correct experimental period
	ReportLine <- paste(TubeDetailsDat[1,"Directory"],
		gsub(".txt","",TubeDetailsDat[1,"Monitor"]),
		as.character(ResetDat[1,"date"]),as.character(ResetDat[1,"time"]),
		as.character(ResetDat[nrow(ResetDat),"date"]),
		as.character(ResetDat[nrow(ResetDat),"time"]),
		(nrow(ResetDat)/1440),sep="\t")
	write(ReportLine,AdHocReporting,append=T)

    # CODE each light (dark) period activity bin as "L" ("D")
    # CODE each bin by the L or D period (1:N)
    # GET number of light and dark periods
    LightDat <- LightStatusCode(zbTime)
    numLightPeriods <- max(as.numeric(LightDat[,2])[LightDat[,1]=="L"])
    numDarkPeriods <- max(as.numeric(LightDat[,2])[LightDat[,1]=="D"])

	# GENERATE sleep (1) / awake (0) vectors => matrix
	# RENAME columns
	SleepDat <- GetSleepMatrix(ResetDat)
	colnames(SleepDat) <- paste("SLEEP",names(RIL),RIL,
		TubeDetailsDat[,"Genotype"],
		TubeDetailsDat[,"Experimenter"],sep=".")

	# GENERATE common set of columns for output
	CommonCols <- cbind(zbTime,LightDat,ResetDat[,"light.status"])
	colnames(CommonCols)[1:2] <- c("ZBhrs","ZBmin")
	colnames(CommonCols)[5] <- c("Monitor.LightStatus")

	# EXTRACT activity columns and rename
	ActivityDat <- ResetDat[,10:41]
	colnames(ActivityDat) <- paste("ACT",names(RIL),RIL,
		TubeDetailsDat[,"Genotype"],
		TubeDetailsDat[,"Experimenter"],sep=".")

	# LOOP through unique RILs on monitor
	# OUTPUT data per RIL
	#   -> CommonCols / ActivityDat (for RIL) / Sleep Dat (for RIL)
	
	for(j in 1:length(uniqueRIL)) {
		# MATCH current RIL against positions of RILs in monitor
		# EXTRACT information on this RIL from 'TubeDetailsDat'
		# GET barcode (BC) and experimenter details
		matchRIL <- match(RIL,uniqueRIL[j],nomatch=0)
		RILTubeDetailsDat <- TubeDetailsDat[matchRIL==1,]
		outputBC <- as.numeric(unique(RILTubeDetailsDat[,"Genotype"]))
		outputExperimenter <- unique(RILTubeDetailsDat[,"Experimenter"])
		
		# GENERATE output data for the current RIL
		# EDIT rownames (to begin from 1)
		outputData <- cbind(CommonCols,ActivityDat[,matchRIL==1],
			SleepDat[,matchRIL==1])
		rownames(outputData) <- 1:nrow(outputData)
		
		# OUTPUT it
		outputName <- paste(dirProcessed,uniqueRIL[j],".",outputBC,
			".",outputExperimenter,".txt",sep="")
		write.table(outputData,outputName,col.names=NA,sep="\t",quote=F)
	}
}



##########################################################################################
### ANALYSIS
##########################################################################################

# READ in names of all processed, per-RIL sleep data files
# EXTRACT genotype numbers
SleepDataFiles <- list.files(dirProcessed,pattern="txt")
#GenoList <- as.numeric(substr(SleepDataFiles,1,5))
GenoList <- substr(SleepDataFiles,1,13)  # CHANGED to extract full cross information
UniqueGeno <- unique(GenoList)

# SET NA threshold, above which a monitor tube will not
# be used
thresholdNA <- 0

# SET output data files
SampleSize.Output <- NULL
LightSleepTime.Output <- NULL
DarkSleepTime.Output <- NULL
TotalWakingActivity.Output <- NULL
LightWakingActivity.Output <- NULL
DarkWakingActivity.Output <- NULL
LightBoutNumber.Output <- NULL
DarkBoutNumber.Output <- NULL
LightBoutTime.Output <- NULL
DarkBoutTime.Output <- NULL

# LOOP through all genotypes
for(i in 1:length(UniqueGeno)) {

	# DETERMINE which files to read in
	ToReadIn <- SleepDataFiles[match(GenoList,UniqueGeno[i],nomatch=0)==1]
	
	# READ in files
	# IF 1 file - simply read in
	# IF >1 file - read in first 1, the append ACT/SLEEP columns to it
	if(length(ToReadIn)==1) {
		CurDat <- read.table(paste(dirProcessed,ToReadIn,sep=""),header=T)
	}
	else {
		for(j in 1:length(ToReadIn)) {
			if(j==1) {
				CurDat <- read.table(paste(dirProcessed,ToReadIn[1],sep=""),header=T)
			}
			else {
				tmpDat <- read.table(paste(dirProcessed,ToReadIn[j],sep=""),header=T)
				CurDat <- cbind(CurDat,tmpDat[6:21])
			}
		}
	}
	
	# DETERMINE which columns in input file hold ACTIVITY and SLEEP data
	ActivityCols <- grep("ACT.",colnames(CurDat))
	SleepCols <- grep("SLEEP.",colnames(CurDat))
	
	# EXTRACT light/dark data separately
	LightDat <- CurDat[CurDat[,"LightCode"]=="L",]
	DarkDat <- CurDat[CurDat[,"LightCode"]=="D",]
	
	# FIND any columns (equivalent to a single animal in a tube)
	# containing NA values
	# USE threshold to convert this to a 0/1 vector
	# where 0 means a "good" tube and 1 means a "bad" tube
	# that won't be used further
	NumNAvaluesPerTube <- as.numeric(apply(CurDat[,ActivityCols],2,function(XXX)
		length(XXX[is.na(XXX)])))
	NumNAvaluesPerTube[NumNAvaluesPerTube>thresholdNA] <- 1
	
	# BASED on this vector get sample size
	SampleSize <- length(NumNAvaluesPerTube[NumNAvaluesPerTube==0])
	
	# USE this vector to generate sleep and activity datafiles
	# for downstream analyses that lack columns with above
	# threshold numbers of NA values
	allACTIVITY <- CurDat[,ActivityCols]
	allACTIVITY <- allACTIVITY[,NumNAvaluesPerTube==0]
	lightACTIVITY <- LightDat[,ActivityCols]
	lightACTIVITY <- lightACTIVITY[,NumNAvaluesPerTube==0]
	darkACTIVITY <- DarkDat[,ActivityCols]
	darkACTIVITY <- darkACTIVITY[,NumNAvaluesPerTube==0]
	allSLEEP <- CurDat[,SleepCols]
	allSLEEP <- allSLEEP[,NumNAvaluesPerTube==0]
	lightSLEEP <- LightDat[,SleepCols]
	lightSLEEP <- lightSLEEP[,NumNAvaluesPerTube==0]
	darkSLEEP <- DarkDat[,SleepCols]
	darkSLEEP <- darkSLEEP[,NumNAvaluesPerTube==0]
	
	# GENERATE vectors encoding the light/dark period
	# for every row in file
	LDperiod.all <- paste(CurDat[,"LightCode"],CurDat[,"CountCode"],sep="")
	LDperiod.light <- paste(LightDat[,"LightCode"],LightDat[,"CountCode"],sep="")
	LDperiod.dark <- paste(DarkDat[,"LightCode"],DarkDat[,"CountCode"],sep="")
	
	# SLEEP estimates
	LightSleepTime <- CalcSleepTime(lightSLEEP,LDperiod.light)
	DarkSleepTime <- CalcSleepTime(darkSLEEP,LDperiod.dark)

	# WAKING ACTIVITY estimates
	TotalWakingActivity <- WakingActivity(allACTIVITY,allSLEEP)
	LightWakingActivity <- WakingActivity(lightACTIVITY,lightSLEEP)
	DarkWakingActivity <- WakingActivity(darkACTIVITY,darkSLEEP)
	
	# SLEEP BOUT estimates
	LightBouts <- GetBoutData(lightSLEEP,LDperiod.light)
	DarkBouts <- GetBoutData(darkSLEEP,LDperiod.dark)

	# OUTPUT all data into summary files
	SampleSize.Output <- c(SampleSize.Output,SampleSize)
	LightSleepTime.Output <- rbind(LightSleepTime.Output,LightSleepTime)
	DarkSleepTime.Output <- rbind(DarkSleepTime.Output,DarkSleepTime)
	TotalWakingActivity.Output <- rbind(TotalWakingActivity.Output,TotalWakingActivity)
	LightWakingActivity.Output <- rbind(LightWakingActivity.Output,LightWakingActivity)
	DarkWakingActivity.Output <- rbind(DarkWakingActivity.Output,DarkWakingActivity)
	LightBoutNumber.Output <- rbind(LightBoutNumber.Output,LightBouts[1:4])
	DarkBoutNumber.Output <- rbind(DarkBoutNumber.Output,DarkBouts[1:4])
	LightBoutTime.Output <- rbind(LightBoutTime.Output,LightBouts[5:8])
#	LightBoutTime.Output <- rbind(LightBoutTime.Output,sqrt(LightBouts[5:8]))
#	LightBoutTime.Output <- rbind(LightBoutTime.Output,log10(LightBouts[5:8]))
#	LightBoutTime.Output <- rbind(LightBoutTime.Output,(1/sqrt(LightBouts[5:8])))
#	LightBoutTime.Output <- rbind(LightBoutTime.Output,(1/(LightBouts[5:8])^2))
	DarkBoutTime.Output <- rbind(DarkBoutTime.Output,DarkBouts[5:8])
#	DarkBoutTime.Output <- rbind(DarkBoutTime.Output,sqrt(DarkBouts[5:8]))
#	DarkBoutTime.Output <- rbind(DarkBoutTime.Output,log10(DarkBouts[5:8]))
}

# CLEAN UP output data files
# OUTPUT
CommonColNames <- c("N","Mean","Median","SD","SE")
CommonRowNames <- UniqueGeno

LightSleepTime.Output <- cbind(SampleSize.Output,LightSleepTime.Output)
colnames(LightSleepTime.Output) <- CommonColNames
rownames(LightSleepTime.Output) <- CommonRowNames
write.table(LightSleepTime.Output,paste(dirSummary,"LightSleepTime.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
DarkSleepTime.Output <- cbind(SampleSize.Output,DarkSleepTime.Output)
colnames(DarkSleepTime.Output) <- CommonColNames
rownames(DarkSleepTime.Output) <- CommonRowNames
write.table(DarkSleepTime.Output,paste(dirSummary,"DarkSleepTime.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
TotalWakingActivity.Output <- cbind(SampleSize.Output,TotalWakingActivity.Output)
colnames(TotalWakingActivity.Output) <- CommonColNames
rownames(TotalWakingActivity.Output) <- CommonRowNames
write.table(TotalWakingActivity.Output,paste(dirSummary,"TotalWakingActivity.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
LightWakingActivity.Output <- cbind(SampleSize.Output,LightWakingActivity.Output)
colnames(LightWakingActivity.Output) <- CommonColNames
rownames(LightWakingActivity.Output) <- CommonRowNames
write.table(LightWakingActivity.Output,paste(dirSummary,"LightWakingActivity.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
DarkWakingActivity.Output <- cbind(SampleSize.Output,DarkWakingActivity.Output)
colnames(DarkWakingActivity.Output) <- CommonColNames
rownames(DarkWakingActivity.Output) <- CommonRowNames
write.table(DarkWakingActivity.Output,paste(dirSummary,"DarkWakingActivity.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
LightBoutNumber.Output <- cbind(SampleSize.Output,LightBoutNumber.Output)
colnames(LightBoutNumber.Output) <- CommonColNames
rownames(LightBoutNumber.Output) <- CommonRowNames
write.table(LightBoutNumber.Output,paste(dirSummary,"LightBoutNumber.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
DarkBoutNumber.Output <- cbind(SampleSize.Output,DarkBoutNumber.Output)
colnames(DarkBoutNumber.Output) <- CommonColNames
rownames(DarkBoutNumber.Output) <- CommonRowNames
write.table(DarkBoutNumber.Output,paste(dirSummary,"DarkBoutNumber.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
LightBoutTime.Output <- cbind(SampleSize.Output,LightBoutTime.Output)
colnames(LightBoutTime.Output) <- CommonColNames
rownames(LightBoutTime.Output) <- CommonRowNames
#write.table(LightBoutTime.Output,paste(dirSummary,"LightBoutTime1-Y2.txt",sep=""),
#write.table(LightBoutTime.Output,paste(dirSummary,"LightBoutTime1-SQRT.txt",sep=""),
#write.table(LightBoutTime.Output,paste(dirSummary,"LightBoutTimeLOG10.txt",sep=""),
#write.table(LightBoutTime.Output,paste(dirSummary,"LightBoutTimeSQRT.txt",sep=""),
write.table(LightBoutTime.Output,paste(dirSummary,"LightBoutTime.txt",sep=""),
	quote=F,col.names=NA,sep="\t")
DarkBoutTime.Output <- cbind(SampleSize.Output,DarkBoutTime.Output)
colnames(DarkBoutTime.Output) <- CommonColNames
rownames(DarkBoutTime.Output) <- CommonRowNames
write.table(DarkBoutTime.Output,paste(dirSummary,"DarkBoutTime.txt",sep=""),
#write.table(DarkBoutTime.Output,paste(dirSummary,"DarkBoutTimeSQRT.txt",sep=""),
#write.table(DarkBoutTime.Output,paste(dirSummary,"DarkBoutTimeLOG10.txt",sep=""),
	quote=F,col.names=NA,sep="\t")



##########################################################################################
### DATA INTERROGATION
##########################################################################################

# THE principal reason for doing this inbred/outbred DSPR experiment was
#   because of observation that in DSPRF1 sleep time (ESPECIALLY light
#   sleep time) seemed to be much less than in DGRP. Thought perhaps
#   due to inbreeding. And testing in DSPR.

# GET mean genotype dark/light sleep time for inbred/outbred animals
# (need to run material below twice after commenting in/out the "DAT" line)
#
# GET female and male parental genotypes
# GENERATE inbred (1), outbred (0) vector
# EXTRACT mean pheno per genotype
DAT <- DarkSleepTime.Output
#DAT <- LightSleepTime.Output
Fparent <- as.numeric(substr(rownames(DAT),1,5))
Mparent <- as.numeric(substr(rownames(DAT),8,12))
InOut <- (Fparent==Mparent)*1
meanPheno <- DAT[,"Mean"]

		# LIGHT SLEEP TIME
		#> t.test(meanPheno[InOut==1],meanPheno[InOut==0])
		#
		#	Welch Two Sample t-test
		#
		#data:  meanPheno[InOut == 1] and meanPheno[InOut == 0]
		#t = 1.0773, df = 63.757, p-value = 0.2854
		#alternative hypothesis: true difference in means is not equal to 0
		#95 percent confidence interval:
		# -25.62238  85.58601
		#sample estimates:
		#mean of x mean of y 
		# 169.3066  139.3248
		#
		# SO while sleep time less in outbreds, not (overall)
		# significantly so
		
		# DARK SLEEP TIME
		#> t.test(meanPheno[InOut==1],meanPheno[InOut==0])
		#
		#	Welch Two Sample t-test
		#
		#data:  meanPheno[InOut == 1] and meanPheno[InOut == 0]
		#t = -0.4414, df = 57.95, p-value = 0.6606
		#alternative hypothesis: true difference in means is not equal to 0
		#95 percent confidence interval:
		# -73.15506  46.72149
		#sample estimates:
		#mean of x mean of y 
		# 469.6342  482.8509
		#
		# NO difference

# GENERATE table holding the pheno of each outbred genotype,
#   along with the phenotypes of its inbred parents
OutbredRow <- (1:nrow(DAT))[InOut==0]
InOutPheno <- NULL
for(i in 1:length(OutbredRow)) {
	# PHENO of outbred genotype
	outPheno <- DAT[OutbredRow[i],"Mean"]
	# CROSS FORMAT for both inbred parent genotypes
	FparCross <- paste(Fparent[OutbredRow[i]],"F.",Fparent[OutbredRow[i]],"M",sep="")
	MparCross <- paste(Mparent[OutbredRow[i]],"F.",Mparent[OutbredRow[i]],"M",sep="")
	# MATCH these to 'DAT' rownames and extract inbred phenotypes
	inF.pheno <- DAT[match(FparCross,rownames(DAT)),"Mean"]
	inM.pheno <- DAT[match(MparCross,rownames(DAT)),"Mean"]
	# CONCATENATE all 3 values
	# ALONG with inbred parent mean
	InOutPheno <- rbind(InOutPheno,c(outPheno,inF.pheno,inM.pheno,(inF.pheno+inM.pheno)/2))
	rownames(InOutPheno)[nrow(InOutPheno)] <- rownames(DAT)[OutbredRow[i]]
}
colnames(InOutPheno) <- c("outMean","inF.Mean","inM.Mean","inMid.Mean")

# REMOVE any rows with NA values (where didn't get phenotypes from both inbred
#   parents)
InOutPheno.trunc <- InOutPheno[as.numeric(apply(InOutPheno,1,function(XXX) length(XXX[is.na(XXX)])))==0,]

# HOW many outbred genotypes tested? (SAME for light and dark sleep time)
#
#> nrow(InOutPheno.trunc)
#[1] 30

# HOW often is the outbred mean below the means of both inbred parents?
#
#> sum((InOutPheno.trunc[,1]<InOutPheno.trunc[,2] & InOutPheno.trunc[,1]<InOutPheno.trunc[,3])*1)
#[1] 10   FOR LIGHT sleep time
#[1]  6   FOR DARK sleep time

# HOW often is the outbred mean below the inbred parent mid-point?
#
#> sum((InOutPheno.trunc[,1]<InOutPheno.trunc[,4])*1)
#[1] 22   FOR LIGHT sleep time
#[1] 14   FOR DARK sleep time

# SORT 'InOutPheno.trunc' based on outbred phenotype
InOutPheno.trunc.sort <- InOutPheno.trunc[order(InOutPheno.trunc[,1]),] 

# PUBLICATION QUALITY FIGURE
# FIRST run stuff about and make Lsleep and Dsleep plot objects
Dsleep <- InOutPheno.trunc.sort
Lsleep <- InOutPheno.trunc.sort

pdf(paste(dirSummary,"SuppFig3_InOutbred_DSPR.pdf",sep=""),width=6,height=8)
#quartz(width=6,height=8)
par(mfrow=c(2,1))
baroffset <- 0.2
# LIGHT
DATASET <- Lsleep
# Main plot outline
plot(DATASET[,1],pch=16,ylim=c(0,720),main="Light",type="n",
	las=1,ylab="Sleep time (mins)",xlab="DSPR Genotypes")
# Bars going between M and F inbred parents
# Female lines
# Male lines
# Outbred crosses
for(i in 1:nrow(DATASET)) {
	rect(i-baroffset,DATASET[i,2],i+baroffset,DATASET[i,3],
		col="gray83",border=NA)
	lines(c(i-baroffset,i+baroffset),c(DATASET[i,2],DATASET[i,2]),
		col="orchid2",lwd=2)
	lines(c(i-baroffset,i+baroffset),c(DATASET[i,3],DATASET[i,3]),
		col="lightslateblue",lwd=2)
	points(i,DATASET[i,1],pch=4,col="black")
}
legend("topleft",pch=c(15,15,4),col=c("orchid2","lightslateblue","black"),
	c("Inbred (F parent)","Inbred (M parent)","Outbred cross"),bty="n",
	cex=0.8)

# LIGHT
DATASET <- Dsleep
# Main plot outline
plot(DATASET[,1],pch=16,ylim=c(0,720),main="Dark",type="n",
	las=1,ylab="Sleep time (mins)",xlab="DSPR Genotypes")
# Bars going between M and F inbred parents
# Female lines
# Male lines
# Outbred crosses
for(i in 1:nrow(DATASET)) {
	rect(i-baroffset,DATASET[i,2],i+baroffset,DATASET[i,3],
		col="gray83",border=NA)
	lines(c(i-baroffset,i+baroffset),c(DATASET[i,2],DATASET[i,2]),
		col="orchid2",lwd=2)
	lines(c(i-baroffset,i+baroffset),c(DATASET[i,3],DATASET[i,3]),
		col="lightslateblue",lwd=2)
	points(i,DATASET[i,1],pch=4,col="black")
}
graphics.off()



































##########################################################################################
##########################################################################################
### PASTE IN ALL FUNCTIONS BEFORE
### BEFORE RUNNING ANY CODE ABOVE
##########################################################################################
##########################################################################################


###################################
# FUNCTION
###################################
# GENERIC, simple function return set
# of summary stats (number, mean, median,
# SD) for a vector of input values
###################################

SummarizeValues <- function(X) {
    X <- as.numeric(X)
    N <- length(X[!is.na(X)])
    meanX <- mean(X,na.rm=T)
    medX <- median(X,na.rm=T)
    sdX <- sd(X,na.rm=T)
    return(c(N,meanX,medX,sdX))
}


###################################
# FUNCTION
###################################
# GENERIC function to remove
# NA-containing rows from a matrix
###################################

RemoveRowsWithNAs <- function(FILE) {
    # COUNT number of NA values per row
    tmp <- as.numeric(apply(FILE,1,function(E) length(E[is.na(E)])))
    # KEEP only NA-free rows, output
    Output <- FILE[tmp==0,]
    return(Output)
}


###################################
# FUNCTION
###################################
# READS in data from single monitor (DAT)
# and a zero (or some other value)
# representing the number of alive flies
# per genotype that is acceptable
#
# RETURNS "1" if all flies from that genotype
# are dead, and a "0" otherwise
###################################

AreAllFliesDead <- function(DAT,COUNT) {
    # DETERMINE genotypes represented on monitor
    genoCol <- DAT[,"Genotype"]
    genos <- unique(genoCol)
    genos <- genos[!is.na(genos)]
    numGeno <- length(genos)
    
    # SET temporary live fly coding object
    tmpCode <- rep(NA,numGeno)

    # LOOP through genotypes
    for(i in 1:numGeno) {
        # PULL OUT subset of data associated with single genotype
        # COUNT number of "alive" values in 'AliveOrDead' column
        subDAT <- DAT[DAT[,"Genotype"]==genos[i] & !is.na(DAT[,"Genotype"]),]
        AliveOrDead <- subDAT[,"AliveOrDead"]
        numAlive <- length(AliveOrDead[AliveOrDead=="alive" & !is.na(AliveOrDead)])

        # IF more "alive" than threshold ('COUNT') return 0, otherwise return 1
        if(numAlive>COUNT) {
            tmpCode[i] <- 0
        }
        else {
            tmpCode[i] <- 1
        }
        # GENERATE vector showing that block of tubes from same
        # monitor holding same genotype all died ("1") or not ("0")
        genoCol[genoCol==genos[i] & !is.na(genoCol)] <- tmpCode[i]
    }
    
    # RETURN
    genoCol[genoCol!=0 & genoCol!=1] <- NA
    OUTPUT <- matrix(genoCol,ncol=1)
    colnames(OUTPUT) <- "AllDead"
    rownames(OUTPUT) <- names(genoCol)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READS in single arbitrary barcode,
# and 2-column RIL duplication matrix
#
# RETURNS associated RIL barcode
###################################

FindRILBarcode <- function(X,BARCODES) {
    # SIMPLE match function
    # -> assumes 2-column RIL duplication
    #    file with 1st col = RILbc and 2nd
    #    col = ARBbc of duplicated vial
    RILBC <- BARCODES[match(X,BARCODES[,2]),1]
    return(RILBC)
}


###################################
# FUNCTION
###################################
# READS in single element of 'AllMonitorDat'
# (data from one monitor)
#
# RETURNS summary information from each genotype
# on that monitor
###################################

SummarizeMonitorData <- function(DAT) {
    # EXTRACT directory/monitor info
    curDir <- DAT[1,"Directory"]
    curMon <- DAT[1,"Monitor"]
    
    # DETERMINE genotypes represented on monitor
    genos <- unique(DAT[,"Genotype"])
    genos <- genos[!is.na(genos)]
    numGeno <- length(genos)
    
    # MAKE output matrix based on this information
    OutputMat <- matrix(rep(NA,(numGeno*11)),ncol=11)
    colnames(OutputMat) <- c("Directory","Monitor","Genotype","RIL",
        "Experimenter","N","N.alive","N.dead","MeanLifespan",
        "MedianLifespan","SDLifespan")
        
    # LOOP through genotypes
    for(i in 1:numGeno) {
    
        # FILL in directory/monitor info
        OutputMat[i,1] <- curDir
        OutputMat[i,2] <- curMon
       
        # EXTRACT just tubes for given genotype
        # GET current genotype
        # GET associated RIL barcode
        # GET current experimenter
        subDAT <- DAT[DAT[,"Genotype"]==genos[i] & !is.na(DAT[,"Genotype"]),]
        curGeno <- subDAT[1,"Genotype"]
        OutputMat[i,3] <- curGeno
        OutputMat[i,4] <- subDAT[1,"RIL"]
        OutputMat[i,5] <- subDAT[1,"Experimenter"]
        
        # EXTRACT lifespan data
        # GET N, mean, median, SD
        lifeDAT <- as.numeric(subDAT[,"Lifespan.hours"])
        lifeDAT <- lifeDAT[!is.na(lifeDAT)]
        OutputMat[i,6] <- length(lifeDAT)
        OutputMat[i,9] <- mean(lifeDAT)
        OutputMat[i,10] <- median(lifeDAT)
        OutputMat[i,11] <- sd(lifeDAT)
        
        # EXTRACT alive/dead data
        # GET N alive and N dead
        adDAT <- subDAT[,"AliveOrDead"]
        adDAT <- adDAT[!is.na(adDAT)]
        OutputMat[i,7] <- length(adDAT[adDAT=="alive"])
        OutputMat[i,8] <- length(adDAT[adDAT=="dead"])
    }
    return(OutputMat)
}


###################################
# FUNCTION
###################################
# GENERIC function to read in files
# of same type (given the path to the
# directory) and output a single file
# concatenating all input files using
# 'rbind'
###################################

ReadInFiles.func <- function(PATH) {
    FileNames <- list.files(path=PATH)
    Output <- NULL
    for(i in FileNames) {
        tmpFile <- read.csv(paste(PATH,i,sep=""),header=T,sep="\t")
        Output <- rbind(Output,tmpFile)
    }
    return(Output)
}


###################################
# FUNCTION
###################################
# READ in monitor data, a test window
# at the end of the file (SPAN in bins)
# to check for activity, and an acceptable
# number of counts to consider the fly
# dead
#
# RETURN matrix of "alive" or "dead"
# or "NA"
###################################

FliesAreAlive <- function(DAT,SPAN,THRESH) {
    # TRANSFORM just date/time columns into something R likes
    AllDateTime <- apply(DAT[,1:2],1,function(bbb) paste(bbb[1],bbb[2],sep=" "))

    # EXTRACT the last date/time point at which the monitor
    #   had a "status" of 1 (i.e., not 24 or 51)
    # GENERATE date/time SPAN minutes before
    tmpADT <- AllDateTime[DAT[,"status"]==1]
    lastTime <- as.character(tmpADT[length(tmpADT)])
    firstTime <- strptime(lastTime,"%Y-%m-%d %H:%M:%S")-(SPAN*60)
    
    # GENERATE submatrix of 'DAT' between the first and last times
    endDAT <- DAT[AllDateTime>firstTime & AllDateTime<=lastTime,]
    # ACTIVITY counts in this period
    # VECTOR determining whether entire period is NA's (then return
    # NA value)
    # MAKE a vector stating whether fly is alive or not (based on
    #   input activity threshold)
    # RETURN as matrix
    endCounts <- apply(endDAT[,10:41],2,function(X) sum(X,na.rm=T))
    naCol <- apply(endDAT[,10:41],2,function(Y)
        if(length(Y[is.na(Y)])==length(Y)) {return(NA)} else {return(1)})
    aliveVector <- endCounts
    aliveVector[endCounts<=THRESH] <- "dead"
    aliveVector[endCounts>THRESH] <- "alive"
    aliveVector[is.na(naCol)] <- NA
    OUTPUT <- matrix(aliveVector,ncol=1)
    colnames(OUTPUT) <- "AliveOrDead"
    rownames(OUTPUT) <- names(aliveVector)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in tube details matrix and death
#   data matrix, and desired units of
#   returned time difference data
#
# RETURNS vector of times (1 per tube)
###################################

Lifespan <- function(TUBE,DEATH,UNITS) {
    # EXTRACT just the time data from the input matrices
    startTime <- TUBE[,"StartTimestamp"]
    deathTime <- DEATH[,"TimeOfDeath"]
    # GET difference between then in defined units
    # TURN into 1-column matrix and return
    TimeDiff <- apply(cbind(deathTime,startTime),1,function(AA)
        as.numeric(difftime(AA[1],AA[2],units=UNITS)))
    OUTPUT <- matrix(TimeDiff,ncol=1)
    colnames(OUTPUT) <- paste("Lifespan",UNITS,sep=".")
    rownames(OUTPUT) <- names(TimeDiff)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in current row of 'MonitorInfo'
#   file, and a FORMAT
# Possible 'FORMAT' values are "2halves"
#   (meaning using A/B barcodes) or
#   "4quadrants" (using 1/2/3/4 barcodes)
# RETURNS a matrix defining the monitor
#   details (directory/monitor), starting
#   time, genotype, and experimenter
#   for each tube
###################################

PopulateIDMatrix <- function(X,FORMAT) {
    # BOTH monitor sides filled with different genotypes
    # (potentially at different times by different people)
    if(FORMAT=="2halves") {
        # EXTRACT information
        TheDir <- X["DAMdir"]
        TheMon <- X["MonFname"]
        timeA <- X["Timestamp.A"]
        genoA <- X["Genotype.A"]
        exptA <- X["Experimenter.A"]
        timeB <- X["Timestamp.B"]
        genoB <- X["Genotype.B"]
        exptB <- X["Experimenter.B"]
        # DEFINE tube positions for A and B sides
        TubeData <- c("B","B","B","B","A","A","A","A",
            "B","B","B","B","A","A","A","A",
            "B","B","B","B","A","A","A","A",
            "B","B","B","B","A","A","A","A")
        # DEVELOP categorization vectors
        rDir <- rep(TheDir,32)
        rMon <- rep(TheMon,32)
        rTime <- TubeData
        rTime[TubeData=="A"] <- timeA
        rTime[TubeData=="B"] <- timeB
        rGeno <- TubeData
        rGeno[TubeData=="A"] <- genoA
        rGeno[TubeData=="B"] <- genoB
        rExpt <- TubeData
        rExpt[TubeData=="A"] <- exptA
        rExpt[TubeData=="B"] <- exptB
        # GENERATE matrix
        OUT <- cbind(rDir,rMon,rTime,rGeno,rExpt)
        colnames(OUT) <- c("Directory","Monitor","StartTimestamp","Genotype","Experimenter")
        rownames(OUT) <- names(X)[3:34]
    }
    else {
        # EXTRACT information
        TheDir <- X["DAMdir"]
        TheMon <- X["MonFname"]
        time1 <- X["Timestamp.1"]
        geno1 <- X["Genotype.1"]
        expt1 <- X["Experimenter.1"]
        time2 <- X["Timestamp.2"]
        geno2 <- X["Genotype.2"]
        expt2 <- X["Experimenter.2"]
        time3 <- X["Timestamp.3"]
        geno3 <- X["Genotype.3"]
        expt3 <- X["Experimenter.3"]
        time4 <- X["Timestamp.4"]
        geno4 <- X["Genotype.4"]
        expt4 <- X["Experimenter.4"]
        # DEFINE tube positions for 1, 2, 3, and 4 quadrants
        TubeData <- c("1","1","2","2","3","3","4","4",
            "1","1","2","2","3","3","4","4",
            "1","1","2","2","3","3","4","4",
            "1","1","2","2","3","3","4","4")
        # DEVELOP categorization vectors
        rDir <- rep(TheDir,32)
        rMon <- rep(TheMon,32)
        rTime <- TubeData
        rTime[TubeData=="1"] <- time1
        rTime[TubeData=="2"] <- time2
        rTime[TubeData=="3"] <- time3
        rTime[TubeData=="4"] <- time4
        rGeno <- TubeData
        rGeno[TubeData=="1"] <- geno1
        rGeno[TubeData=="2"] <- geno2
        rGeno[TubeData=="3"] <- geno3
        rGeno[TubeData=="4"] <- geno4
        rExpt <- TubeData
        rExpt[TubeData=="1"] <- expt1
        rExpt[TubeData=="2"] <- expt2
        rExpt[TubeData=="3"] <- expt3
        rExpt[TubeData=="4"] <- expt4
        # GENERATE matrix
        OUT <- cbind(rDir,rMon,rTime,rGeno,rExpt)
        colnames(OUT) <- c("Directory","Monitor","StartTimestamp","Genotype","Experimenter")
        rownames(OUT) <- names(X)[3:34]
    }
    return(OUT)
}


###################################
# FUNCTION
###################################
# READ in DAT (ValidDat - where unwanted
# monitor tubes should be NA'd out), DEATH
# (DeathDat), current monitor info row,
# and an output directory for plot
#
# MAKES single PDF per monitor (with
# individual plots per tube)
#
# Activity data simply a barplot, with the
# lifespan determined by my code as a red
# line, and the last activity bin as a blue
# triangle
###################################

ActivityBarplot8x4 <- function(DAT,DEATH,MINFO,OUTDIR) {
    # GENERATE plotfile output name
    # OPEN plot
    # LOOP through tubes in 'DAT'
    nameOut <- paste(OUTDIR,MINFO[1],".",gsub("txt","pdf",MINFO[2]),sep="")
    pdf(nameOut,width=25,height=13.8)
    pX <- matrix(c(1,9,17,25,2,10,18,26,3,11,19,27,4,12,20,28,5,13,21,29,6,
        14,22,30,7,15,23,31,8,16,24,32),ncol=4,byrow=T)
    layout(pX)
    for(i in 10:41) {
        plotDAT <- DAT[,i]
        # ONLY make plot if column not full of NA values
        if(length(plotDAT[is.na(plotDAT)])<length(plotDAT)) {
            # ENSURE the lifespan line always in relatively the same
            # position along x-axis
            maxDat <- max(plotDAT,na.rm=T)
            LifeDuration <- -maxDat/30
            # Activity barplot
            par(mar=c(2.5,5,1,1))
            X <- barplot(plotDAT,las=1,ylab="Activity Per Bin")
            mtext(colnames(DAT)[i],side=1,line=1)
            # Lifespan line
            lines(c(X[1],X[as.numeric(DEATH[(i-9),2])]),c(LifeDuration,LifeDuration),
                col="red",lwd=3,xpd=T)
            # Last active bin triangle
            points(X[max((1:length(plotDAT))[!is.na(plotDAT) & plotDAT>0])],(LifeDuration*2.5),
                pch=17,col="blue",cex=1,xpd=T)
        }
        else {
            par(mar=c(2.5,5,1,1))
            barplot(0,ylim=c(0,20),las=1,ylab="Activity Per Bin")
            mtext(colnames(DAT)[i],side=1,line=1)
        }
    }
    graphics.off()
}

ActivityBarplot1x32 <- function(DAT,DEATH,MINFO,OUTDIR) {
    # GENERATE plotfile output name
    # OPEN plot
    # LOOP through tubes in 'DAT'
    nameOut <- paste(OUTDIR,MINFO[1],".",gsub("txt","pdf",MINFO[2]),sep="")
    pdf(nameOut,width=8,height=4)
    for(i in 10:41) {
        plotDAT <- DAT[,i]
        # ONLY make plot if column not full of NA values
        if(length(plotDAT[is.na(plotDAT)])<length(plotDAT)) {
            # ENSURE the lifespan line always in relatively the same
            # position along x-axis
            maxDat <- max(plotDAT,na.rm=T)
            LifeDuration <- -maxDat/30
            # Activity barplot
            X <- barplot(plotDAT,las=1,ylab="Activity Per Bin",
                main=paste("./ ",MINFO[1]," / ",MINFO[2],sep=""),
                xlab=colnames(DAT)[i])
            # Lifespan line
            lines(c(X[1],X[as.numeric(DEATH[(i-9),2])]),c(LifeDuration,LifeDuration),
                col="red",lwd=4,xpd=T)
            # Last active bin triangle
            points(X[max((1:length(plotDAT))[!is.na(plotDAT) & plotDAT>0])],(LifeDuration*2),
                pch=17,col="blue",cex=1,xpd=T)
        }
        else {
            # DO NOTHING
        }
    }
    graphics.off()
}


###################################
# FUNCTION
###################################
# READ in DAT (ValidDat - where unwanted
# monitor tubes should be NA'd out), RI/RIU
# (read interval and units), and two arbitrary
# parameters to be passed to 'TubeDeathTime'
# function. Function essentially just
# wrapper for this other function.
#
# DOES generate the date/timepoint of the
# last status=1 row in the file (basically
# point at which monitor unplugged or
# stopped recording activity data)
#
# RETURNS matrix of time of death per tube
###################################

DeathTimes <- function(DAT,RI,RIU,MNSIB,MNAB) {
    # EXTRACT the date/time columns
    # EXTRACT just the monitor tube activity data
    DTVECT <- DAT[,1:2]
    ACTDAT <- DAT[,10:41]
    
    # APPLY FUNCTION through the tube data
    TimeOfDeath <- matrix(apply(ACTDAT,2,function(X)
        TubeDeathTime(X,DTVECT,RI,RIU,MNSIB,MNAB)),ncol=3,byrow=T)
    colnames(TimeOfDeath) <- c("TimeOfDeath","RowDeath","NumBinsRemoved")
    rownames(TimeOfDeath) <- c(paste("m0",seq(1,9,1),sep=""),paste("m",seq(10,32,1),sep=""))
    return(TimeOfDeath)
}


###################################
# FUNCTION
###################################
# PROVIDE validated input monitor file
# and row of 'MonitorInfo' file pertaining
# to current file
#
# RETURNS monitor file with any tubes having
# a (bad) 0 value in row turned to an NA vector
###################################

RemoveBadTubes <- function(DAT,MROW) {
    # EXTRACT just the tubes' 1(good)/0(bad) codes
    # EXTRACT names of any tubes that are bad
    tCodes <- MROW[3:34]
    badTubes <- names(tCodes[tCodes=="0"])
    # IF no bad tubes, do nothing, otherwise
    # NA out those columns in the monitor file
    if(length(badTubes)==0) {
        # DO NOTHING
    }
    else {
        DAT[,match(badTubes,colnames(DAT))] <- NA
    }
    return(DAT)
}


###################################
# FUNCTION
###################################
# PROVIDE path to DAM initiation file directory,
#   the text in the stem of each initiation file,
#   and the text in the stem of each DAM output
#   file directory
#
# ONLY possibilities for 'FORMAT' are "2halves"
#   and "4quadrants"
#
# RETURNS a file with one line per monitor (over
#   entire expt) including the directory where the
#   monitor file is, its filename, whether both
#   A and B sides were used, timestamps, genotypes,
#   and experimenters for each side
###################################

CollateInitiationFiles <- function(IDIR,INIT.STEM,DIR.STEM,FORMAT) {
    # LIST files in directory
    fileList <- list.files(IDIR)
    # SET an output file
    OUTPUT <- NULL

    # LOOP through and read in / process files
    for(i in 1:length(fileList)) {

        # READ in and sort by "Monitor.Barcode" column
        # GENERATE name of DAM output file directory associated
        #   with this initiation file
        curFile <- read.delim(paste(IDIR,fileList[i],sep=""),header=T)
        curFile <- curFile[order(curFile[,3]),]
        DamFileDirectory <- gsub(INIT.STEM,DIR.STEM,fileList[i])
        DamFileDirectory <- gsub(".txt","",DamFileDirectory)
        
        # GET sorted list of all monitors used
        mColumn <- as.character(curFile[,"Monitor.Barcode"])
        mSides <- unlist(lapply(mColumn,function(X)
            strsplit(X,split=".",fixed=T)[[1]][1]))
        mUsed <- sort(unique(mSides))

        # LOOP through them
        for(j in 1:length(mUsed)) {

            # GENERATE monitor filename (= "Monitor1.txt", etc)
            MonitorFilename <- paste("Monitor",as.numeric(strsplit(mUsed[j],
                split="M",fixed=T)[[1]][2]),".txt",sep="")

            # EXTRACT rows of 'curFile' containing data from given monitor
            # EXTRACT "tail" of barcode (will be A or B, or 1, 2, 3, or 4)
            monRows <- curFile[match(mSides,mUsed[j],nomatch=0)==1,]
            bcTail <- substr(monRows[,3],5,5)
            
            # GENERATE tube data (1=good, 0=bad/absent) based on
            # known loading pattern
            # -> Set vector of zeros
            # -> Fill in depending on 'bcTail' value
            TubeData <- rep(0,32)
            for(k in 1:length(bcTail)) {
                if(bcTail[k]=="A") {
                    TubeData[c(5:8,13:16,21:24,29:32)] <- rep(1,16)
                }
                if(bcTail[k]=="B") {
                    TubeData[c(1:4,9:12,17:20,25:28)] <- rep(1,16)
                }
                if(bcTail[k]=="1") {
                    TubeData[c(1,2,9,10,17,18,25,26)] <- rep(1,8)
                }
                if(bcTail[k]=="2") {
                    TubeData[c(3,4,11,12,19,20,27,28)] <- rep(1,8)
                }
                if(bcTail[k]=="3") {
                    TubeData[c(5,6,13,14,21,22,29,30)] <- rep(1,8)
                }
                if(bcTail[k]=="4") {
                    TubeData[c(7,8,15,16,23,24,31,32)] <- rep(1,8)
                }
            }

            # EXTRACT timestamps, genotypes, and experimenters
            Timestamp <- apply(monRows,1,function(X) TimestampConvertExcelToR(X[1]))
            Genotypes <- monRows[,2]
            Experimenters <- as.character(monRows[,4])
            
            # SOLVE PROBLEM
            # -> Possible that monitor not full, and need to fill
            #    correct timestamp, genotype, experimentor into
            #    correct cells of OUTPUT table
            if(FORMAT=="2halves") {
                if(length(bcTail)==2) {
                    # DO NOTHING
                }
                else {
                    if(bcTail=="A") {
                        Timestamp <- c(Timestamp,NA)
                        Genotypes <- c(Genotypes,NA)
                        Experimenters <- c(Experimenters,NA)
                    }
                    else {
                        Timestamp <- c(NA,Timestamp)
                        Genotypes <- c(NA,Genotypes)
                        Experimenters <- c(NA,Experimenters)
                    }
                }
            }
            else {
                if(length(bcTail)==4) {
                    # DO NOTHING
                }
                else {
                    Timestamp <- Timestamp[match(1:4,bcTail)]
                    Genotypes <- Genotypes[match(1:4,bcTail)]
                    Experimenters <- Experimenters[match(1:4,bcTail)]
                }
            }
            # COLLATE all data for current monitor (i.e., value of 'mUsed')
            OUTPUT <- rbind(OUTPUT,c(DamFileDirectory,MonitorFilename,TubeData,Timestamp,
                Genotypes,Experimenters))
        }
    }
    # ADD column names and return 'OUTPUT'
    if(FORMAT=="2halves") {
        colnames(OUTPUT) <- c("DAMdir","MonFname",paste("m0",seq(1,9,1),sep=""),
            paste("m",seq(10,32,1),sep=""),paste("Timestamp",c("A","B"),sep="."),
            paste("Genotype",c("A","B"),sep="."),paste("Experimenter",c("A","B"),sep="."))
    }
    else {
        colnames(OUTPUT) <- c("DAMdir","MonFname",paste("m0",seq(1,9,1),sep=""),
            paste("m",seq(10,32,1),sep=""),paste("Timestamp",1:4,sep="."),
            paste("Genotype",1:4,sep="."),paste("Experimenter",1:4,sep="."))
        
    }
    return(OUTPUT)
}
            

###################################
# FUNCTION
###################################
# READ Excel-based timestamp from
#   monitor initiation file
#
# RETURN an R-formated version
###################################

TimestampConvertExcelToR <- function(IN) {
    # CONVERT Excel-based timestamp in R flavor
    OUT <- strptime(as.character(IN),"%m/%d/%y %H:%M")
    OUT <- as.character(OUT)
    # PROBLEM: if timestamp exactly at midnight, R fails
    #   to put on time portion - do it here
    if(nchar(OUT)==19) {
        # DO NOTHING - this is correct
    }
    else {
        OUT <- paste(OUT,"00:00:00",sep=" ")
    }
    return(OUT)
}


###################################
# FUNCTION
###################################
# READ in validated monitor file (a 'ValidDat'
#   file), and list of columns that hold bad
#   data that you want to remove (or "None"
#   if no columns are bad)
#
# RETURN file with any columns of bad data
#   eliminated (converted to NA vector)
###################################

RemoveLowActivityTubeData <- function(MON,BADCOLS) {
    # IF NO 'bad' columns, simply return input 'MON' file
    if(BADCOLS[1]=="None") {
        # DO NOTHING
    }
    # OTHERWISE convert those columns to NA vectors
    else {
        MON[,match(BADCOLS,colnames(MON))] <- NA
    }
    # RETURN matrix
    return(MON)
}


###################################
# FUNCTION
###################################
# READ in validated monitor file (a 'ValidDat'
#   file), and a minimal acceptable activity
#   threshold (lower, and assume no fly or its
#   very sick)
# RETURN names of any such tubes
###################################

FindLowActivityTubes <- function(MON,ACTHRESH) {
    # CALCULATE total activity from each tube
    sumAct <- apply(MON[,10:41],2,function(X) sum(X,na.rm=T))
    # EXTRACT names of tubes with less than some
    # threshold number of activity counts
    lowActNames <- names(sumAct[sumAct<ACTHRESH])
    # RETURN these tube names (or "None")
    if(length(lowActNames)>0) {
        return(lowActNames)
    }
    else {
        return("None")
    }
}


###################################
# FUNCTION
###################################
# READ in binned activity counts from 1 tube (XACT),
#    the date/time columns from the binned datafile (DTVECT),
#    the bin interval and units (BI,BIU), and a threshold (TB)
#    giving the max number of inactive bins that can occur between
#    active bins before we decide the later activity is spurious, and
#    a number of active bins that can occur following the 'TB' sized
#    hiatus in activity ('BFE')
#
# -> The actual value of 'TB' is purely ad hoc
#
# OUTPUTs time/date of death for tube, and a 0/1 value showing
#    whether potentially spurious active bins were ignored
#
# NOTE: If a fly is alive when monitor is unplugged AND it is active
#       in the last bin in 'ValidDat' function will return an "NA"
#       for 'rowDeath' since the row-of-death is generated via a
#       match function as that row AFTER the fly shows last activity
#       count (and such a row won't exist in this case)
#       Did NOT solve problem since only result is to fail to give
#       red "lifespan" line on output plot. AND in any case these
#       individuals are alive, so the data from that genotype gets
#       ignored.
###################################

# NOTE:
# Might need to incorporate some additional thresholds: For instance,
#   one that only ignores late-occurring activity bins if the count is
#   very small, or if the late bins are isolated (they don't get
#   ignored if there are multiple late active bins

TubeDeathTime <- function(XACT,DTVECT,BI,BIU,TB,BFE) {
    # CHECK if 'XACT' holds (at least some) activity counts
    #   (if entire vector is NAs just return NA values)
    if(length(XACT[is.na(XACT)])<length(XACT)) {
        # GATHER only dates/times of bins for which activity
        #   count is >0
        # TRANSFORM into vector format R can work with
        #   [rm. any NA values that got passes in 'XACT' - could be
        #    result of selecting an experimental data window that
        #    includes bad monitor reads (that are NA'd out)]
        DTact <- DTVECT[XACT>0 & !is.na(XACT),]
        DTactv <- apply(DTact,1,function(AAA) paste(AAA[1],AAA[2],sep=" "))
    
        # GENERATE vector of # bins between each non-0 activity
        #   count bin (e.g., a 1 means fly is active in consecutive bins)
        #   -> vector is padded with a 0 at start
        BinSplit <- 0
        for(i in 1:(length(DTactv)-1)) {
            BinSplit <- c(BinSplit,difftime(DTactv[i+1],DTactv[i],units=BIU))
        }
        
        # CHECK whether any values in 'BinSplit' are greater than
        # input value of 'TB'
        if(sum((BinSplit>TB)*1,na.rm=T)==0) {
            # IF NO don't change date vector
            DTactv.t <- DTactv
        }
        else {
            # CHECK whether any value in final 'BFE' set of 'BinSplit'
            # values is >'TB'
            BinSplitEnd <- BinSplit[(length(BinSplit)-BFE+1):length(BinSplit)]
            EndOverTB <- match(1,(BinSplitEnd>TB)*1)
            if(is.na(EndOverTB)==TRUE) {
                # IF NO don't change date vector
                DTactv.t <- DTactv
            }
            else {
                # IF YES truncate date vector (so last value is time/date of
                # last activity BEFORE long inactive period)
                EndOverTB <- EndOverTB+length(BinSplit)-BFE
                DTactv.t <- DTactv[1:(EndOverTB)-1]
            }
        }
        
        # TEST whether any activity bins were removed
        if(length(DTactv)==length(DTactv.t)) {
            rmBin <- 0
        }
        else {
            rmBin <- 1
        }
        
        # EXTRACT the time of the bin that follows the one in which
        #   the fly was last active
        #   -> first define length of a bin
        #   -> also deal with potential to hit midnight, when R stops putting time
        #      onto output
        AddBin <- TimePeriodConversion(BI,BIU,"secs")
        TimeDeath <- as.character(strptime(as.character(DTactv.t[length(DTactv.t)]),
            "%Y-%m-%d %H:%M:%S")+AddBin)
        if(nchar(TimeDeath)==19) {
            # DO NOTHING
        }
        else {
            TimeDeath <- paste(TimeDeath,"00:00:00",sep=" ")
        }
        
        # EXTRACT the row of input 'DTVECT' corresponding to 'TimeDeath'
        reformatDTVECT <- apply(DTVECT,1,function(BBB) paste(BBB[1],BBB[2],sep=" "))
        rowDeath <- match(TimeDeath,reformatDTVECT)
        
        # OUTPUT time of death, row of death, and whether bins were removed
        return(c(TimeDeath,rowDeath,rmBin))
    }
    # IF input data all NAs
    else {
        return(c(NA,NA,NA))
    }
}


###################################
# FUNCTION
###################################
# READ in reformatted monitor datafile, 
#    the expected time between reads ('INTERVAL'),
#    the units of this time ('UNIT', one of
#    "secs", "mins", or "hours"), the current row
#    of the monitor info file, and the path to the
#    error file
# REMOVES read lines with exact duplicate date/time
#    stamps
# WRITES to error file if the monitor file includes
#   anything other then a set of sequential rows
#   each differing by INTERVAL UNIT time
# RETURNS monitor file
###################################

CheckDupsIntervalsGaps <- function(DAT,INTERVAL,UNIT,MROW,OUT) {
    # EXTRACT date and time from DAT into 2-col
    # 'as.character' matrix
    # REMOVE date and time columns from DAT
    DT <- cbind(DAT[,1],as.character(DAT[,2]))
    DAT <- DAT[,3:ncol(DAT)]

    # CHECK for exact duplicate date/time stamps
    # REMOVE any duplicates from two matrices
    Xdup <- duplicated(DT)*1
    DT2 <- DT[Xdup==0,]
    DAT2 <- DAT[Xdup==0,]

    # CALCULATE time difference between sequential reads
    nReads <- nrow(DAT2)
    DT2.both <- paste(DT2[,1],DT2[,2],sep=" ")
    intReads <- as.numeric(difftime(DT2.both[2:nReads],
        DT2.both[1:(nReads-1)],),units=UNIT)
    
    # CODE time differences (0 = interval expected, -1 = less,
    #    +1 = double, +2 = more, but not exactly double)
    # COUNT instances of the two types of gap
    intReadsCode <- c(as.numeric(lapply(intReads,function(A)
        if(A==INTERVAL) {return(0)} else {if(A==(2*INTERVAL))
        {return(1)} else {if(A>INTERVAL & A!=(2*INTERVAL))
        {return(2)} else {return(-1)}}})),0)
    countm1 <- length(intReadsCode[intReadsCode==-1])
    countp1 <- length(intReadsCode[intReadsCode==1])
    count2 <- length(intReadsCode[intReadsCode==2])
    
    # GENERATE error count line
    errorLine <- paste(MROW[1],MROW[2],countm1,countp1,count2,sep="\t")
    write(errorLine,OUT,append=T)
    
    # OUTPUT
    OUTPUT <- cbind(DT2,DAT2)
    colnames(OUTPUT)[1:2] <- c("date","time")
    rownames(OUTPUT) <- 1:nrow(OUTPUT)
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in a value, and two time unit character
#    objects ("secs","mins","hours")
# CONVERT value from time in input units
#    to time in output units
###################################

TimePeriodConversion <- function(NUM,INUNIT,OUTUNIT) {
    # SET unit possibilities
    Upos <- c("secs","mins","hours")
    # CHECK both input units are correct
    # otherwise return NULL
    Ucheck <- sum(c(INUNIT==Upos,OUTUNIT==Upos))
    if(Ucheck!=2) {
        Output <- NULL
    }
    else {
        # GENERATE matrix of all possible combinations of
        # input and output units
        Ugrid <- expand.grid(Upos,Upos)
        # GENERATE vector of unit conversions that matches
        # format of matrix above
        Uconv <- c(1,60,3600,(1/60),1,60,(1/3600),(1/60),1)
        
        # FIND row of matrix that matches the input/output
        # unit values
        matchI <- match(Ugrid[,1],INUNIT,nomatch=0)
        matchO <- match(Ugrid[,2],OUTUNIT,nomatch=0)
        RowMatch <- which.max(matchI+matchO)
        
        # RETURN input value multipled by conversion factor
        Output <- NUM * Uconv[RowMatch]
    }
    return(Output)
}


###################################
# FUNCTION
###################################
# READ in monitor datafile
# CHECK it has correct num columns (via 'NC')
# ELIMINATE data for bad monitor reads
# STRIP unwanted/unused columns (via 'STRPCOLS')
# ADD column names
# REFORMAT date column to something more
#     familiar for R
###################################

InputMonitorFile <- function(FNAME,NC,STRPCOLS) {
    DAT <- read.delim(file=FNAME,header=F)
    # CHECK that number of columns of file is correct
    ncDAT <- ncol(DAT)
    if(ncDAT!=NC) {
        return(NULL)
    }
    else {
        # DEAL with 'bad' monitor reads
        # (convert data in all columns except
        #  index, date, time, and status to NA)
        DAT[DAT[,4]!=1,5:NC] <- rep(NA,(NC-4))
        
        # DEAL with possibility want to strip unused/unwanted
        # columns or not
        if(STRPCOLS=="yes") {
            # STRIP unwanted columns (keep date/time/monitor reads)
            DAT <- DAT[,-c(1,4:10)]
            # ADD useful column names
            colnames(DAT) <- c("date","time",
                paste(rep("m",9),1:9,sep="0"),
                paste(rep("m",23),10:32,sep=""))
        }
        if(STRPCOLS=="no") {
            # REMOVE index column only
            DAT <- DAT[,-1]
            # ADD useful column names
            colnames(DAT) <- c("date","time","status","extra.readings",
                "unused1","unused2","unused3","unused4","light.status",
                paste(rep("m",9),1:9,sep="0"),
                paste(rep("m",23),10:32,sep=""))
        }
        # REFORMAT date and output
        DAT[,1] <- as.character(strptime(DAT[,1],"%d %b %y"))
        return(DAT)
    }
}


###################################
# FUNCTION
###################################
# READ in 'ValidDat'
# OUTPUT new form of file so first row is
# the first period of data collection
###################################

TrimStartMonitorFile <- function(VDAT) {
    # EXTRACT 4th column 'extra.readings' which is
    #   NA unless the monitor is collecting data
	ExtraReadings <- VDAT[,4]
	# LOOP through this vector until find a non-NA
	#   value; This is the start of data collection
	StartDataRow <- 0
	for(i in 1:length(ExtraReadings)) {
		if(is.na(ExtraReadings[i]) & is.na(ExtraReadings[i+1])) {
			# DO NOTHING
		}
		else {
			StartDataRow <- i+1
			break
		}
	}
	# TRIM start of input file
	OUTPUT <- VDAT[StartDataRow:nrow(VDAT),]
	return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in 'ValidDat'
# OUTPUT new form of file so last row is
# the last period of data collection
###################################

TrimEndMonitorFile <- function(VDAT) {
    # EXTRACT 4th column 'extra.readings' which is
    #   NA unless the monitor is collecting data
    # REVERSE it (since targeting end of file)
	ExtraReadings <- rev(VDAT[,4])
	# LOOP through this vector until find a non-NA
	#   value; This is the start of data collection
	EndDataRow <- 0
	for(i in 1:length(ExtraReadings)) {
		if(is.na(ExtraReadings[i]) & is.na(ExtraReadings[i+1])) {
			# DO NOTHING
		}
		else {
			EndDataRow <- length(ExtraReadings)-i
			break
		}
	}
	# TRIM start of input file
	OUTPUT <- VDAT[1:EndDataRow,]
	return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in 'ValidDat' and 'DeathDat' and a value
# equal to the number of hours before fly death
# you wish to eliminate activity data
# FOR EACH tube set all activity bins
# 'PHRS' hours before fly death (and all bins post
# fly death) to NA
# OUTPUT new form of 'ValidDat'
#
# NOTE: Assumes 1min reading interval used
###################################

ResetAfterDeathActivityCounts <- function(VDAT,DDAT,PHRS) {
    # OBJECT to hold output data
    OUTDAT <- VDAT
    # LOOP through monitor tubes
    # NOTE that the '9' in the next few rows
    #    skips over non-monitor columns in VDAT
    for(i in 1:nrow(DDAT)) {
    	# SET current monitor tube activity data
        tmpActVect <- VDAT[,(i+9)]
        # IF fly didn't die during experiment,
        # OR 'died' in last hour, keep current activity data
        if(is.na(DDAT[i,2]) | (nrow(VDAT)-as.numeric(DDAT[i,2])<=60)) {
            OUTDAT[,(i+9)] <- tmpActVect
        }
        else {
        	# IF fly dies
            # SET bins 'PHRS' hours before death (and after death) to NA
            # (The 1st two lines ensure don't try to NA out positions
            #  in vector pre-1)
            PosToNAOut <- (as.numeric(DDAT[i,2])-(PHRS*60)):length(tmpActVect)
            PosToNAOut <- PosToNAOut[PosToNAOut>0]
            tmpActVect[PosToNAOut] <- NA
            OUTDAT[,(i+9)] <- tmpActVect
        }
    }
    return(OUTDAT)
}


###################################
# FUNCTION
###################################
# SET zeitgeber time in light and dark
# in 1 minute intervals
# L = 0.1--0.60,1.1-1.60,..11.60
# D = 12.1--12.60,13.1-13.60,..23.60
###################################

LightZeitgeber.H <- sort(rep(seq(0,11,1),60))
LightZeitgeber.M <- rep(1:60,12)
DarkZeitgeber.H <- sort(rep(seq(12,23,1),60))
DarkZeitgeber.M <- rep(1:60,12)


###################################
# FUNCTION
###################################
# READ in 'ValidDat' (or 'ResetDat') along with
# manually-generated (see above) Zeitgeber time
# vectors
# RETURN vector of these timestamps for
# every row of input file
#
# NOTE that this code currently assumes that
# data has been collected on a minute-by-minute
# basis (i.e., readInterval = 1 and readIntUnits = "mins")
###################################

GetZietgeberTime <- function(VDAT,LZH,LZM,DZH,DZM) {
    # EXTRACT date/time of 1st L-D (or D-L) switch
    # row1=last light (or dark) bin
    # row2=first dark (or light) bin
    # (NOTE: 1=L,0-D)

    # EXTRACT time at first light-dark, dark-light switch
    TIMESWITCH <- NULL
    for(i in 2:nrow(VDAT)) {
        # IF either sequential timestamp = NA
        if(is.na(VDAT[i,"light.status"]) | is.na(VDAT[(i-1),"light.status"])) {
            # DO NOTHING
        }
        else {
            # IF sequential timestamps are the same
            if(VDAT[i,"light.status"]==VDAT[(i-1),"light.status"]) {
                # DO NOTHING
            }
            else {
                # IF sequential timestamps different
                TIMESWITCH <- rbind(TIMESWITCH,VDAT[(i-1),c(1,2,9)],
                    VDAT[i,c(1,2,9)])
                break
            }
        }
    }
    
    # SET switch as L->D (1->0) or D->L (0->1)
    if(TIMESWITCH[1,3]==1) {
        LIGHTVECTOR.H <- c(DZH,LZH)  # for Light->Dark transition
        LIGHTVECTOR.M <- c(DZM,LZM)  # for Light->Dark transition
    }
    else {
        LIGHTVECTOR.H <- c(LZH,DZH)  # for Dark->Light transition
        LIGHTVECTOR.M <- c(LZM,DZM)  # for Light->Dark transition
    }
    
    # SET number of rows of input, validated monitor file
    rowsVD <- nrow(VDAT)

    # CALCULATE zeitgeber time for all rows in input matrix
    # OUTPUT
    zg1.H <- LIGHTVECTOR.H[(1440-i+2):1440]                         # pre-1st switch point
    zg2.H <- rep(LIGHTVECTOR.H,floor((rowsVD-i+1)/1440))            # after 1st switch point (whole days)
    zg3.H <- LIGHTVECTOR.H[1:(rowsVD-length(zg1.H)-length(zg2.H))]  # after 1st switch point (remainder)
    ZeitgeberTimeStamps.H <- c(zg1.H,zg2.H,zg3.H)
    zg1.M <- LIGHTVECTOR.M[(1440-i+2):1440]                       # pre-1st switch point
    zg2.M <- rep(LIGHTVECTOR.M,floor((rowsVD-i+1)/1440))          # after 1st switch point (whole days)
    zg3.M <- LIGHTVECTOR.M[1:(rowsVD-length(zg1.M)-length(zg2.M))]  # after 1st switch point (remainder)
    ZeitgeberTimeStamps.M <- c(zg1.M,zg2.M,zg3.M)
    return(cbind(ZeitgeberTimeStamps.H,ZeitgeberTimeStamps.M))
}


###################################
# FUNCTION
###################################
# READ in 2-col 'zbTime' matrix
# RETURN matrix describing each bin
# as in Light or Dark, and the number
# each of Light and Dark period
###################################

LightStatusCode <- function(ZBT) {
    # VECTOR coding light status up to first dark period
    # VECTOR coding light period count up to first dark period
    LightCode <- rep("L",(match(12,ZBT[,1])-1)) # Assumes monitors set up in light
    CountCode <- rep(1,(match(12,ZBT[,1])-1))

    # COMPLETE vector coding light status
    numBins <- nrow(ZBT)
    numDays <- ceiling(numBins/1440)
    LightCode <- c(LightCode,rep(c(rep("D",720),rep("L",720)),numDays))
    LightCode <- LightCode[1:nrow(ZBT)]

    # COMPLTE vector coding light/dark period
    Dcount <- 1
    Lcount <- 2
    for(i in 1:numDays) {
        CountCode <- c(CountCode,rep(Dcount,720),rep(Lcount,720))
        Dcount <- Dcount+1
        Lcount <- Lcount+1
    }
    CountCode <- CountCode[1:nrow(ZBT)]
    return(cbind(LightCode,CountCode))
}


###################################
# FUNCTION
###################################
# READ in 'ResetDat' form of monitor data
#
# RETURN matrix of 32 columns (1/monitor tube)
#   where each value is either NA (no info),
#   1 (fly asleep), 0 (fly awake)
#
# NOTE: Sleep/Awake determined based on 5min bins
#
# NOTE: Code assumes have collected data in 1min bins
#       and have filtered the 'RDAT' input to be a
#       perfect number of 5min bins
###################################

GetSleepMatrix <- function(RDAT) {

	# EXTRACT only data from 32 monitor tubes
	DAT <- RDAT[,10:41]

    # GENERATE start and stop rows for each 5 minute window
    # NOTE assumes data collected in 1min bins
    # AND further assumes the data provided to this function
    #   has already been trimmed to be binnable in this way
    FiveMinWindows <- cbind(seq(1,(nrow(DAT)-4),5),
        seq(5,nrow(DAT),5))
    
	# 	
	SleepMatrix <- NULL
	for(i in 1:nrow(FiveMinWindows)) {
		# EXTRACT start/end of current 5min window
		# GET vectors of sleep for this window
		#     NA = no information
		#     1  = fly is asleep
		#     0  = fly is awake
		XXX <- FiveMinWindows[i,1]
		YYY <- FiveMinWindows[i,2]
		tmpMat <- apply(DAT[XXX:YYY,],2,function(AAA) ReturnSleepVector(AAA))
		SleepMatrix <- rbind(SleepMatrix,tmpMat)
	}
	return(SleepMatrix)
}


###################################
# FUNCTION
###################################
# READ in single, short vector of values
# (activity counts)
# RETURN vector of the same length;
# -> All NA values = at least 1 input element = NA
# -> All  1 values = all input elements = 0
# -> All  0 values = sum of input elements >0
###################################

ReturnSleepVector <- function(DAT) {
    # GET length of 'DAT'
	# SUM values in 'DAT'
    lDAT <- length(DAT)
    sDAT <- sum(DAT)
    
    # IF sum = 0 - fly is asleep, return vector of 1's
    # IF sum > 0 - fly is awake,  return vector of 0's
    # IF sum = NA (i.e., one or more of the input values = NA)
    #            - have no info,  return vector of NA's
    if(sDAT==0 & !is.na(sDAT)) {
        OUTPUT <- rep(1,lDAT)
    }
    else {
    	if(sDAT>0 & !is.na(sDAT)) {
    		OUTPUT <- rep(0,lDAT)
    	}
    	else {
    		OUTPUT <- rep(NA,lDAT)
    	}
    }
    return(OUTPUT)
}


###################################
# FUNCTION
###################################
# READ in matrix of columns of sleep data
# VALUES can be 1 (asleep) or 0 (awake)
#
# READ in light/dark 12hr period vector
#
# NOTE that function currently assumes that NA
# is NOT a possible value
#
# NOTE that function currently assumes 1 min
# data collection intervals (such that the number
# of 12hr periods in the data is 720 rows)
#
# RETURNS mean number of sleep bouts per 12hr period
# RETURNS mean length of sleep bouts
###################################

GetBoutData <- function(DAT,LD) {
	# GET vector of unique 12hr periods
	uLD <- unique(LD)
	# LOOP through each period and individual
	# CALL outside function to get data on sleep bouts
	NumSleepBouts <- NULL
	AllSleepBouts <- NULL
	for(i in 1:length(uLD)) {
		subDAT <- DAT[LD==uLD[i],]
		for(j in 1:ncol(subDAT)) {
			BoutInfo <- BoutInfoForSingleSleepVector(subDAT[,j])
			# STORE number of sleep bouts
			# COLLATE info on all sleep bouts
			NumSleepBouts <- c(NumSleepBouts,nrow(BoutInfo))
			AllSleepBouts <- rbind(AllSleepBouts,BoutInfo)
		}
	}
	# GET summary statistics on bout number
	MeanNumSleepBouts <- mean(NumSleepBouts)
	MedianNumSleepBouts <- median(NumSleepBouts)
	SDNumSleepBouts <- sd(NumSleepBouts)
	SENumSleepBouts <- SDNumSleepBouts/sqrt(length(NumSleepBouts))
	# GET summary statistics on bout length
	AllBoutLength <- AllSleepBouts[,2]-AllSleepBouts[,1]+1
	MeanLength <- mean(AllBoutLength)
	MedianLength <- median(AllBoutLength)
	SDLength <- sd(AllBoutLength)
	SELength <- SDLength/sqrt(length(AllBoutLength))
	# RETURN
	return(c(MeanNumSleepBouts,MedianNumSleepBouts,SDNumSleepBouts,SENumSleepBouts,
		MeanLength,MedianLength,SDLength,SELength))
}


###################################
# FUNCTION
###################################
# READ in 0/1 vector (0 = awake, 1 = asleep)
#
# RETURNS 2-col matrix of start/endpoints of
# all sleep bouts
#
# CANNOT deal with NA values in input vector
###################################

BoutInfoForSingleSleepVector <- function(VECT) {
	# GENERATE objects for storing start/end of sleep bouts
	StartBout <- NULL
	EndBout <- NULL
	
	# LOOP through every element in vector
	# NA = no info, 1=sleep, 0=awake
	
	# FIRST element requires special consideration
	# IF already sleeping, set as start of bout
	if(!is.na(VECT[1]) & VECT[1]==1) {
		StartBout <- 1
	}
	else {
		# DO NOTHING
	}
	for(i in 2:length(VECT)) {
		# DO NOTHING IF
		#    CURRENT/PREVIOUS both 1 (asleep)
		#    CURRENT/PREVIOUS both 0 (awake)
		#    CURRENT/PREVIOUS both NA (no info)
		#    CURRENT is 0 and PREVIOUS is NA
		#    CURRENT is 1 and PREVIOUS is NA
		#    CURRENT is NA and PREVIOUS is 0
		#    CURRENT is NA and PREVIOUS is 1
		
		if((!is.na(VECT[i]) & VECT[i]==1 & !is.na(VECT[i-1]) & VECT[i-1]==1) |
			(!is.na(VECT[i]) & VECT[i]==0 & !is.na(VECT[i-1]) & VECT[i-1]==0) |
			(is.na(VECT[i]) & is.na(VECT[i-1])) |
			(!is.na(VECT[i]) & VECT[i]==0 & is.na(VECT[i-1])) |
			(!is.na(VECT[i]) & VECT[i]==1 & is.na(VECT[i-1])) |
			(is.na(VECT[i]) & !is.na(VECT[i-1]) & VECT[i-1]==0) |
			(is.na(VECT[i]) & !is.na(VECT[i-1]) & VECT[i-1]==1)) {
				# DO NOTHING
		}
		else {
			# IF CURRENT is 1 and PREVIOUS is 0 - this is START of bout
			if((!is.na(VECT[i]) & VECT[i]==1 & !is.na(VECT[i-1]) & VECT[i-1]==0)) {
				StartBout <- c(StartBout,i)
			}
			# OTHERWISE
			# IF CURRENT is 0 and PREVIOUS is 1 - previous was END of bout
			else {
				EndBout <- c(EndBout,(i-1))
			}
		}
	}
	
	# FINAL element requires special consideration
	# IF sleeping, bout has started, but not yet ended
	if(!is.na(VECT[length(VECT)]) & VECT[length(VECT)]==1) {
		EndBout <- c(EndBout,length(VECT))
	}

	# RETURN matrix of start/end bouts
	return(cbind(StartBout,EndBout))
}


###################################
# FUNCTION
###################################
# READ in matrix of columns of activity data
#   NA = no information, # = number of beam crosses per minute
# READ in matrix of columns of sleep data
#   NA = no information, 1 = asleep, 0 = awake
#
# RETURN mean/median/SD/SE of activity/min when awake
###################################

WakingActivity <- function(ACTIVITY,SLEEP) {
	# CONVERT both input matrices to vectors
	vActivity <- as.numeric(unlist(ACTIVITY))
	vSleep <- as.numeric(unlist(SLEEP))

	# CONVERT current sleep vector values (1=sleep,0=awake,NA=no info)
	# to (1=awake, NA=sleep or no info)
	# EXTRACT only activity values when fly is known to be awake
	vSleep <- vSleep+1
	vSleep[vSleep==2] <- NA
	WakingActivity <- vActivity[!is.na(vSleep) & vSleep==1]
	
	# MEAN/MEDIAN/SD activity per waking minute
	# RETURN
	meanWA <- mean(WakingActivity)
	medianWA <- median(WakingActivity)
	sdWA <- sd(WakingActivity)
	seWA <- sdWA/sqrt(length(WakingActivity))
	return(c(meanWA,medianWA,sdWA,seWA))
}


###################################
# FUNCTION
###################################
# READ in matrix of columns of sleep data
# VALUES can be 1 (asleep) or 0 (awake)
#
# READ in light/dark 12hr period vector
#
# NOTE that function currently assumes that NA
# is NOT a possible value
#
# NOTE that function currently assumes 1 min
# data collection intervals (such that the number
# of 12hr periods in the data is 720 rows)
#
# RETURNS mean sleep time (in min) averaging
# over individuals and 12hr periods
###################################

CalcSleepTime <- function(DAT,LD) {
	# GET vector of unique 12hr periods
	uLD <- unique(LD)
	# LOOP through each period and individual
	# and calculate sleep time
	# -> Since sleep is coded by 1 and each
	#    element is 1min can simply sum the
	#    elements in the vector to get sleep time
	SleepPerPeriod <- NULL
	for(i in 1:length(uLD)) {
		subDAT <- DAT[LD==uLD[i],]
		subSleep <- as.numeric(apply(subDAT,2,sum))
		SleepPerPeriod <- rbind(SleepPerPeriod,subSleep)
	}
	# CALCULATE summary values
	# RETURN
	MeanSleep <- mean(SleepPerPeriod)
	MedianSleep <- median(SleepPerPeriod)
	SDSleep <- sd(SleepPerPeriod)
	SESleep <- SDSleep/sqrt(length(uLD)*ncol(DAT))
	return(c(MeanSleep,MedianSleep,SDSleep,SESleep))
}


###########
# QTL Profile LOD plot
# Specifically for A*B crosses
#
# SCANRES - 'DSPRscan' object
# XTHRESH and ATHRESH - numeric values
# PLOTNAME - output name of PDF (must include path)
###########

sjm.AxB.DSPRplot <- function(SCANRES,XTHRESH,ATHRESH,PLOTNAME) {

	# DEFINE maximum y-axis LOD score
	Ymax <- max(SCANRES$LODscores$LOD)
	
	# DEFINE axis text size
	CharExp <- 1.4

	# INITIATE pdf
	pdf(PLOTNAME,height=6,width=14)
	
	# CHROMOSOME X
	# SET size of panel, plot outline of figure
	# PLOT curve
	# ADD box, x- and y-axes, and chromosome label
	# ADD threshold
	qLOD <- SCANRES$LODscores[SCANRES$LODscores[,"chr"]=="X",]

	par(fig=c(0.03/5,1.42/5,0,1))
	plot(qLOD$Gpos,qLOD$LOD,xlab=NA,ylab=NA,
    	cex.axis=CharExp,type="n",axes=FALSE,ylim=c(0,Ymax))
	mtext("LOD",side=2,cex=CharExp,line=3)

	points(qLOD$Gpos,qLOD$LOD,type="l",lwd=2,col="black")

	box() # box
	axis(1,at=c(0,66.2),labels=c(NA,NA),tick=TRUE,cex.axis=CharExp) # x-axis
	axis(1,at=c(0,66.2),labels=c(0,66),tick=FALSE,cex.axis=CharExp,line=0) # x-axis
	axis(2,cex.axis=CharExp,las=1) # y=axis
	text((min(qLOD$Gpos)+2),(Ymax*0.95),
		expression(bold("X")),pos=3,xpd=T,cex=(CharExp*1.1)) # chr label

	abline(h=XTHRESH,lty=2,col="black",lwd=1)

	# CHROMOSOME 2
	# SET size of panel, plot outline of figure
	# PLOT curve
	# ADD box, x- and y-axes, and chromosome label
	# ADD threshold

	qLOD <- SCANRES$LODscores[SCANRES$LODscores[,"chr"]=="2L" | SCANRES$LODscores[,"chr"]=="2R",]

	par(fig=c(1.02/5,3.21/5,0,1),new=TRUE)
	plot(qLOD$Gpos,qLOD$LOD,xlab=NA,ylab=NA,
    	cex.lab=CharExp,type="n",axes=FALSE,ylim=c(0,Ymax))

	points(qLOD$Gpos,qLOD$LOD,type="l",lwd=2,col="black")

	box() # box
	axis(1,at=c(0,54,107.8),labels=c(NA,NA,NA),tick=TRUE,cex.axis=CharExp) # x-axis
	axis(1,at=c(0,54,107.8),labels=c(0,54,108),tick=FALSE,cex.axis=CharExp,line=0) # x-axis
	text((min(qLOD$Gpos)-0.5),(Ymax*0.95),
		expression(bold("2")),pos=3,xpd=T,cex=(CharExp*1.1)) # chr label

	abline(h=ATHRESH,lty=2,col="black",lwd=1)

	# CHROMOSOME 3
	# SET size of panel, plot outline of figure
	# PLOT curve
	# ADD box, x- and y-axes, and chromosome label
	# ADD threshold

	qLOD <- SCANRES$LODscores[SCANRES$LODscores[,"chr"]=="3L" | SCANRES$LODscores[,"chr"]=="3R",]

	par(fig=c(2.81/5,5/5,0,1),new=TRUE)
	plot(qLOD$Gpos,qLOD$LOD,xlab=NA,ylab=NA,
    	cex.lab=CharExp,type="n",axes=FALSE,ylim=c(0,Ymax))

	points(qLOD$Gpos,qLOD$LOD,type="l",lwd=2,col="black")

	box() # box
	axis(1,at=c(0,47.1,103),labels=c(NA,NA,NA),tick=TRUE,cex.axis=CharExp) # x-axis
	axis(1,at=c(0,47.1,103),labels=c(0,47,103),tick=FALSE,cex.axis=CharExp,line=0) # x-axis
	text((min(qLOD$Gpos)-0.5),(Ymax*0.95),
		expression(bold("3")),pos=3,xpd=T,cex=(CharExp*1.1)) # chr label

	abline(h=ATHRESH,lty=2,col="black",lwd=1)
	
	graphics.off()
}
